'----------------------------------------------------------------------------------------
'patterns & practices - Smart Client Software Factory - Guidance Package
'
'This file was generated by this guidance package as part of the solution template
'
'The WorkItemController is an abstract base class that contains a WorkItem. 
'This class contains logic that would otherwise exist in the WorkItem. 
'You can use this class to partition your code between a class that derives from WorkItemController and a WorkItem.
' 
'For more information see: 
'ms-help://MS.VSCC.v90/MS.VSIPCC.v90/ms.practices.scsf.2008apr/SCSF/html/03-01-010-How_to_Create_Smart_Client_Solutions.htm
'
'Latest version of this Guidance Package: http://go.microsoft.com/fwlink/?LinkId=62182
'----------------------------------------------------------------------------------------

Imports Microsoft.Practices.CompositeUI
Imports SmartClientDevelopmentSolution1.Infrastructure.Interface.Services

''' <summary>
''' Base class for a WorkItem controller.
''' </summary>
Public MustInherit Class WorkItemController
    Implements IWorkItemController

    Private _workItem As WorkItem

    ''' <summary>
    ''' Gets or sets the work item.
    ''' </summary>
    ''' <value>The work item.</value>
    <ServiceDependency()> _
    Public Property WorkItem() As WorkItem
        Get
            Return _workItem
        End Get
        Set(ByVal value As WorkItem)
            _workItem = value
        End Set
    End Property

    Public ReadOnly Property ActionCatalogService() As IActionCatalogService
        Get
            Return _workItem.Services.Get(Of IActionCatalogService)()
        End Get
    End Property

	''' <summary>
	''' Creates and shows a smart part on the specified workspace.
	''' </summary>
	''' <typeparam name="TView">The type of the smart part to create and show.</typeparam>
	''' <param name="workspaceName">The name of the workspace in which to show the smart part.</param>
	''' <returns>The new smart part instance.</returns>
	Protected Overridable Function ShowViewInWorkspace(Of TView)(ByVal workspaceName As String) As TView
		Dim view As TView = WorkItem.SmartParts.AddNew(Of TView)()
		WorkItem.Workspaces(workspaceName).Show(view)
		Return view
	End Function

	''' <summary>
	''' Shows a specific smart part in the workspace. If a smart part with the specified id
	''' is not found in the <see cref="WorkItem.SmartParts"/> collection, a new instance
	''' will be created; otherwise, the existing instance will be re used.
	''' </summary>
	''' <typeparam name="TView">The type of the smart part to show.</typeparam>
	''' <param name="viewId">The id of the smart part in the <see cref="WorkItem.SmartParts"/> collection.</param>
	''' <param name="workspaceName">The name of the workspace in which to show the smart part.</param>
	''' <returns>The smart part instance.</returns>
	Protected Overridable Function ShowViewInWorkspace(Of TView)(ByVal viewId As String, ByVal workspaceName As String) As TView
		Dim view As TView

		If WorkItem.SmartParts.Contains(viewId) Then
			view = WorkItem.SmartParts.Get(Of TView)(viewId)
		Else
			view = WorkItem.SmartParts.AddNew(Of TView)(viewId)
		End If

		WorkItem.Workspaces(workspaceName).Show(view)

		Return view
	End Function
	
    Public Overridable Sub Run() Implements IWorkItemController.Run

    End Sub
End Class
